import React, {PropTypes} from 'react'
import {BaseComponent} from '@taylorstine/meteor-base-component'
import {ConfiguredRadium} from '@taylorstine/meteor-radium'
import {InheritedProps} from '@taylorstine/meteor-inherited-props-decorator'
import {PureRender} from '@taylorstine/meteor-pure-render-decorator'
import {makeMeteorDialogStyles} from './meteor-dialog.style.js'
import {makeGetStyles, styleProptype} from "@taylorstine/meteor-util"
import {ScrollController} from '@taylorstine/meteor-scroll-controller'
import {ScrollControllerProducer} from "@taylorstine/meteor-scroll-controller-producer-decorator";
import {ComponentChange, ComponentChangedTypes} from "@taylorstine/meteor-component-change-decorator";
import type {MeteorDialogStyleConfig} from './types'
import {kindProptype} from "@taylorstine/meteor-util";
import {MeteorRipple} from '@taylorstine/meteor-ripple'
import {MeteorSpinner} from '@taylorstine/meteor-spinner'

const getStyles = makeGetStyles(makeMeteorDialogStyles);


export type MeteorDialogProps = {
  styleConfig: MeteorDialogStyleConfig
}

@ScrollControllerProducer({
  ref: '_dialog'
})
@ComponentChange({
  props: {
    open: {
      [ComponentChangedTypes.INITIALIZED]: "_onOpenInitialized",
      [ComponentChangedTypes.CHANGED]: "_onOpenChanged"
    }
  }
})
@ConfiguredRadium({getStyles})
@InheritedProps({getStyles})
@PureRender()
export class MeteorDialog extends BaseComponent {

  _dialog: HTMLElement;

  /**
   * Identifier used to clear the automatic close timeout
   */
  _autoCloseTimeout: any;

  static propTypes = {
    /**
     * The title for the dialog
     */
    title: PropTypes.string,

    /**
     * The message content for the dialog
     */
    message: PropTypes.string,

    /**
     * The function to call when the positive action button is pressed.  Both this and positiveActionTitle must be supplied in order to show a positive action
     */
    positiveAction: PropTypes.func,

    /**
     * The text for the positive action button.  Both this and positiveAction must be supplied in order to show a positive action
     */
    positiveActionTitle: PropTypes.string,
    negativeAction: PropTypes.func,
    negativeActionTitle: PropTypes.string,
    preventCancel: PropTypes.bool,
    onClose: PropTypes.func,
    loading: PropTypes.bool,
    open: PropTypes.bool,
    dialogContentStyles: styleProptype(),
    titleStyles: styleProptype(),
    contentChildren: PropTypes.node,
    preContentChildren: PropTypes.node,
    kind: kindProptype(['product']),
    dialogStyles: styleProptype(),
    styles: styleProptype(),
    styleConfig: PropTypes.object,
    loaderNode: PropTypes.node,

    /**
     * Provided to indicate the dialog should automatically close after opening.  This is the duration the dialog should stay open
     */
    openTime: PropTypes.number,
    TYPE_CHECK: (props:MeteorDialogProps)=>{}
  };

  constructor (props, context) {
    super(props, context);
    this.state = {
      open: props.open
    }
  }

  render () {
    const styles = getStyles(this.props);
    const {open} = this.state;
    const {title, message, positiveAction, positiveActionTitle, negativeAction, negativeActionTitle, loading, children, dialogContentStyles, titleStyles, contentChildren, preContentChildren, dialogStyles, loaderNode, styleConfig} = this.props;

    let positiveActionNode = null;
    if (positiveActionTitle && positiveAction) {
      positiveActionNode = (
        <div style={[styles.dialogAction]} onClick={this._cleanupAction.bind(this, positiveAction)}>
          <p style={[styles.dialogActionTitle, styles.dialogActionTitle.positive]}>{positiveActionTitle}</p>
          <MeteorRipple/>
        </div>
      )
    }

    let negativeActionNode = null;
    if (negativeAction && negativeActionTitle) {
      negativeActionNode = (
        <div style={[styles.dialogAction]} onClick={this._cleanupAction.bind(this, negativeAction)}>
          <p style={[styles.dialogActionTitle, styles.dialogActionTitle.negative]}>{negativeActionTitle}</p>
          <MeteorRipple/>
        </div>
      );
    }

    let contentNode = null;
    if (loading) {
      contentNode = (
        <div style={[styles.dialogContent, styles.loading.dialogContent, dialogContentStyles]}>
          <h6 style={[styles.loading.dialogTitle, titleStyles]}>{title}</h6>
          {loaderNode
            ||
          <MeteorSpinner styleConfig={styleConfig}
                         style={[styles.loading.loader]}
                         active={true}/>
          }
        </div>
      )
    } else {
      contentNode = (
        <div style={[styles.dialogContent, dialogContentStyles]}>
          {title && <h6 style={[styles.dialogTitle, titleStyles]}>{title}</h6>}
          {message && <p style={[styles.dialogMessage]}>{message}</p>}
          {contentChildren}
        </div>
      )
    }
    return (
      <div style={[styles.wrapper, open && styles.active.wrapper]}
           onClick={this._onWrapperClick}
      >
        <div style={[styles.dialog, dialogStyles]} onClick={this._onDialogClick} ref={ref=>this._dialog = ref} className="meteor-dialog">
          <div>
            {preContentChildren}
            {contentNode}
            {children}
            <div style={[styles.dialogActionWrapper]}>
              {negativeActionNode}
              {positiveActionNode}
            </div>
          </div>
        </div>
      </div>
    )
  }

  componentDidUpdate(prevProps, prevState) {
    const {open:lastOpen} = prevState;
    const {open:currentOpen} = this.state;
    if (lastOpen !== currentOpen) {
      this._onOpenStateChanged(currentOpen)
    }
  }

  componentWillUnmount () {
    ScrollController.enableWindowScroll();
    clearTimeout(this._autoCloseTimeout)
  }



  _onOpenStateChanged = (open:boolean)=>{
    const {openTime} = this.props;
    if (openTime) {
      this._autoCloseTimeout = setTimeout(this.close, openTime)
    }
  };



  _onOpenInitialized = (open?: boolean) => {
    if (open) {
      this.open();
    }
  };

  /**Called when the value for the open prop changes**/
  _onOpenChanged = (open) => {
    if (open) {
      this.open()
    } else {
      this.close()
    }
  };

  _onDialogClick = (e) => {
    e.stopPropagation();
  };

  _onWrapperClick = () => {
    const {preventCancel} = this.props;
    if (!preventCancel) {
      this.close();
    }
  };

  _cleanupAction = (action: ?Function) => {
    action && action();
    this.close();
  };

  open= () => {
    ScrollController.disableWindowScroll();
    clearTimeout(this._autoCloseTimeout);
    this.setState({open: true});
  };

  close= () => {
    ScrollController.enableWindowScroll();
    clearTimeout(this._autoCloseTimeout);
    const {onClose} = this.props;
  this.setState({open: false});
  onClose();
  };

  publicApi = {open:this.open,close:this.close}
}

MeteorDialog.defaultProps = {
  onClose: ()=> {},
  positiveAction: ()=> {},
};
